@@ -8,6 +8,7 @@
 #include <KWeatherCore/CAPArea>
 #include <KWeatherCore/CAPParser>
 #include <KWeatherCore/CAPReference>
+#include <KWeatherCore/CAPResource>
 
 #include <QFile>
 #include <QTest>
@@ -34,6 +35,7 @@ private Q_SLOTS:
         QCOMPARE(alert.note(), QString());
 
         QCOMPARE(alert.alertInfos().size(), 1);
+        QCOMPARE(alert.preferredInfoIndexForLocale(), 0);
         const auto &info = alert.alertInfos()[0];
 
         QCOMPARE(info.event(), QLatin1String("SEVERE THUNDERSTORM"));
@@ -45,13 +47,16 @@ private Q_SLOTS:
         QCOMPARE(info.instruction().isEmpty(), false);
         QCOMPARE(info.sender(), QLatin1String("NATIONAL WEATHER SERVICE SACRAMENTO CA"));
         QCOMPARE(info.language(), QLatin1String("en-US"));
+        QVERIFY(!info.language().isEmpty());
         QCOMPARE(info.categories(), KWeatherCore::CAPAlertInfo::Category::Meteorological);
         QCOMPARE(info.urgency(), KWeatherCore::CAPAlertInfo::Urgency::Immediate);
         QCOMPARE(info.severity(), KWeatherCore::CAPAlertInfo::Severity::Severe);
         QCOMPARE(info.certainty(), KWeatherCore::CAPAlertInfo::Certainty::Observed);
         QCOMPARE(info.responseTypes(), KWeatherCore::CAPAlertInfo::ResponseType::Shelter);
         QCOMPARE(info.contact(), QLatin1String("BARUFFALDI/JUSKIE"));
+        QCOMPARE(info.audience(), QString());
 
+        QCOMPARE(info.hasGeometry(), true);
         QCOMPARE(info.areas().size(), 1);
         const auto area = info.areas()[0];
         QCOMPARE(area.description(),
@@ -125,6 +130,7 @@ private Q_SLOTS:
         auto info = alert.alertInfos()[0];
         QCOMPARE(info.language(), QLatin1String("en-US"));
         QCOMPARE(info.areas().size(), 1);
+        QCOMPARE(info.hasGeometry(), true);
         auto area = info.areas()[0];
         QCOMPARE(area.polygons().size(), 0);
         QCOMPARE(area.circles().size(), 1);
@@ -148,6 +154,7 @@ private Q_SLOTS:
 
         QCOMPARE(alert.alertInfos().size(), 1);
         auto info = alert.alertInfos()[0];
+        QCOMPARE(info.hasGeometry(), true);
         QCOMPARE(info.language(), QLatin1String("pt-BR"));
         QCOMPARE(info.areas().size(), 1);
         auto area = info.areas()[0];
@@ -205,6 +212,7 @@ private Q_SLOTS:
 
         QCOMPARE(alert.alertInfos().size(), 3);
         const auto info = alert.alertInfos()[0];
+        QCOMPARE(info.hasGeometry(), true);
         QCOMPARE(info.areas().size(), 1);
         const auto area = info.areas()[0];
         QCOMPARE(area.description(), u"澳門特別行政區行政區域圖");
@@ -223,12 +231,79 @@ private Q_SLOTS:
         QCOMPARE(alert.status(), KWeatherCore::CAPAlertMessage::Status::Actual);
         QCOMPARE(alert.alertInfos().size(), 2);
         const auto info = alert.alertInfos()[0];
+        QCOMPARE(info.hasGeometry(), false);
         QCOMPARE(info.areas().size(), 3);
         const auto area = info.areas()[0];
         QVERIFY(!area.description().isEmpty());
         QCOMPARE(area.geoCodes().size(), 1);
         QCOMPARE(area.polygons().size(), 0);
     }
+
+    void testResources()
+    {
+        QFile f(QFINDTESTDATA("capdata/431022.cap"));
+        QVERIFY(f.open(QFile::ReadOnly));
+        KWeatherCore::CAPParser parser(f.readAll());
+        auto alert = parser.parse();
+
+        QCOMPARE(alert.alertInfos().size(), 1);
+        const auto info = alert.alertInfos()[0];
+
+        QCOMPARE(info.resources().size(), 3);
+        auto res = info.resources()[0];
+        QCOMPARE(res.description(), "Stage 2 Water Restriction Details"_L1);
+        QCOMPARE(res.mimeTypeName(), "application/pdf"_L1);
+        QCOMPARE(res.uri().toString(), "https://alerts.alertable.ca/alerts/attach/21-20250609181149.pdf"_L1);
+        QCOMPARE(res.size(), 370550);
+        QVERIFY(res.hasSize());
+        QCOMPARE(res.digest(), QByteArray::fromHex("d4cff1ae5902b9d1b97164bb07ee20ab59029906"));
+
+        res = info.resources()[1];
+        QCOMPARE(res.mimeTypeName(), "image/png"_L1);
+
+        res = info.resources()[2];
+        QCOMPARE(res.mimeTypeName(), "image/jpeg"_L1);
+        QVERIFY(res.digest().isEmpty());
+        QCOMPARE(res.size(), -1);
+        QVERIFY(!res.hasSize());
+    }
+
+    void testAudience()
+    {
+        QFile f(QFINDTESTDATA("capdata/rs-2.49.0.0.688.0.20250817210003.457535.xml"));
+        QVERIFY(f.open(QFile::ReadOnly));
+        KWeatherCore::CAPParser parser(f.readAll());
+        auto alert = parser.parse();
+
+        QCOMPARE(alert.status(), KWeatherCore::CAPAlertMessage::Status::Actual);
+        QCOMPARE(alert.alertInfos().size(), 3);
+        auto info = alert.alertInfos()[0];
+        QCOMPARE(info.audience(), "Private"_L1);
+
+        QCOMPARE(info.language(), "en-GB"_L1);
+        QCOMPARE(info.languageDisplayName(), "British English"_L1);
+        info = alert.alertInfos()[1];
+        QCOMPARE(info.language(), "sr"_L1);
+        QCOMPARE(info.languageDisplayName(), u"српски");
+        info = alert.alertInfos()[2];
+        QCOMPARE(info.language(), "sr-Latn"_L1);
+        QCOMPARE(info.languageDisplayName(), u"srpski");
+    }
+
+    void testScope()
+    {
+        QFile f(QFINDTESTDATA("capdata/454888.cap"));
+        QVERIFY(f.open(QFile::ReadOnly));
+        KWeatherCore::CAPParser parser(f.readAll());
+        const auto alert = parser.parse();
+
+        QCOMPARE(alert.alertInfos().size(), 1);
+        QCOMPARE(alert.scope(), KWeatherCore::CAPAlertMessage::Scope::Private);
+        QCOMPARE(alert.addresses().size(), 1);
+        QCOMPARE(alert.addresses()[0], "CA1-P-136"_L1);
+        QCOMPARE(alert.codes().size(), 1);
+        QCOMPARE(alert.codes()[0], "profile:CAP-CP:0.4"_L1);
+    }
 };
 
 QTEST_GUILESS_MAIN(CapParserTest)
